package com.caimei365.user.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.caimei365.user.feign.ToolsFeign;
import com.caimei365.user.mapper.MessagePushMapper;
import com.caimei365.user.service.RemoteCallService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 远程调用数据处理
 *
 * @author : Charles
 * @date : 2021/7/15
 */
@Slf4j
@Service
public class RemoteCallServiceImpl implements RemoteCallService {
    @Value("${spring.cloud.config.profile}")
    private String profile;
    @Resource
    private ToolsFeign toolsFeign;
    @Resource
    private MessagePushMapper messagePushMapper;
    /**
     * 请求发短信接口
     *
     * @param markId  跳转类型
     * @param mobile  手机号
     * @param content 内容
     */
    @Override
    public boolean remoteSendSms(int markId, String mobile, String content) {
        AtomicBoolean returnValue = new AtomicBoolean(false);
        try {
            //测试环境手机号允许发短信
            List<String> list = new ArrayList<>();
            list.add("15917362709");
            list.add("15814011616");
            list.add("13100721916");
            list.add("15113936829");
            if ("prod".equals(profile) || list.contains(mobile)) {
                if (StringUtils.isNotBlank(mobile) && mobile.length() == 11) {
                    String regex = "^(1[3-9]\\d{9}$)";
                    Pattern pattern = Pattern.compile(regex);
                    Matcher matcher = pattern.matcher(mobile);
                    if (matcher.matches()) {
                        // 调用 ToolsFeign 发送短信
                        String jsonStr = toolsFeign.getSendSms(mobile, content);
                        JSONObject parseObject = JSONObject.parseObject(jsonStr);
                        if (0 == parseObject.getInteger("code")){
                            if (markId > 0 ) {
                                // 保存短信发送条数+count
                                messagePushMapper.updateSmsSendCount(markId, 1);
                            }
                            returnValue.set(true);
                        }
                    }
                }
            }
        } catch (Exception e) {
            log.error("短信推送异常:", e);
        }
        return returnValue.get();
    }

}
